//
// Copyright (c) 2012, MindFusion LLC - Bulgaria.
//

using System;
using System.Collections.Generic;
using System.Drawing;
using System.Windows.Forms;


namespace MindFusion.Diagramming.WinForms.Samples.CS.Effects
{
	public partial class MainForm : Form
	{
		public MainForm()
		{
			InitializeComponent();

			// Create effects
			diagram.EffectsProvider = new EffectsProvider();
			glassEffect = new GlassEffect();
			aeroEffect = new AeroEffect();
			embossEffect = new EmbossEffect();
			smoothShadowEffect = new SmoothShadowEffect();

			// Defaults
			diagram.DiagramStyle.ShadowBrush = new MindFusion.Drawing.SolidBrush(Color.FromArgb(100, Color.Black));
			diagram.ShadowsStyle = ShadowsStyle.None;

			// Preview shapes
			int i = 0;
			const int size = 20;
			const int perLine = 7;
			foreach (Shape shape in Shape.Shapes)
			{
				if (shape.IsArrowhead)
					continue;
				// BPMN shapes don't add variety
				if (shape.Id.StartsWith("bpmn", StringComparison.InvariantCultureIgnoreCase))
					continue;

				ShapeNode node = diagram.Factory.CreateShapeNode((i % perLine) * (size + 5) + 4,
					(i / perLine) * (size + 5) + 4, size, size);
				node.Shape = shape;
				i++;
			}
			diagram.ResizeToFitItems(4, false);

			// Presets
			defaultPreset = new Preset();
			defaultPreset.Name = "(Default)";
			defaultPreset.UseGlassEffect = false;
			defaultPreset.GlassEffectType = GlassEffectType.Type1;
			defaultPreset.UsePenAsGlow = false;
			defaultPreset.GlowColor = Color.White;
			defaultPreset.ReflectionColor = Color.White;
			defaultPreset.UseAeroEffect = false;
			defaultPreset.Opacity = 40;
			defaultPreset.InnerOutlineColor = Color.White;
			defaultPreset.ShadeColor = Color.Black;
			defaultPreset.UseEmbossEffect = false;
			defaultPreset.UseSmoothShadowEffect = false;
			defaultPreset.DiagramBackground = Color.White;
			defaultPreset.ShapeBorder = Color.DarkSlateGray;
			defaultPreset.ShapeBackground1 = Color.LightSteelBlue;
			defaultPreset.UseShapeBackground2 = false;
			defaultPreset.ShapeBackground2 = Color.White;
			ApplyPreset(defaultPreset);

			Preset preset = new Preset();
			preset.Name = "Glass (Default Colors)";
			preset.UseGlassEffect = true;
			preset.GlassEffectType = GlassEffectType.Type1;
			preset.UsePenAsGlow = false;
			preset.GlowColor = Color.White;
			preset.ReflectionColor = Color.White;
			preset.UseAeroEffect = false;
			preset.Opacity = 40;
			preset.InnerOutlineColor = Color.White;
			preset.ShadeColor = Color.Black;
			preset.UseEmbossEffect = false;
			preset.UseSmoothShadowEffect = false;
			preset.DiagramBackground = Color.White;
			preset.ShapeBorder = Color.DarkSlateGray;
			preset.ShapeBackground1 = Color.LightSteelBlue;
			preset.UseShapeBackground2 = false;
			preset.ShapeBackground2 = Color.White;
			AddToList(preset);

			preset = new Preset();
			preset.Name = "Glass & Aero (Orange Background)";
			preset.UseGlassEffect = true;
			preset.GlassEffectType = GlassEffectType.Type2;
			preset.UsePenAsGlow = false;
			preset.GlowColor = Color.White;
			preset.ReflectionColor = Color.White;
			preset.UseAeroEffect = true;
			preset.Opacity = 20;
			preset.InnerOutlineColor = Color.White;
			preset.ShadeColor = Color.Black;
			preset.UseEmbossEffect = false;
			preset.UseSmoothShadowEffect = false;
			preset.DiagramBackground = Color.Orange;
			preset.ShapeBorder = Color.DarkSlateGray;
			preset.ShapeBackground1 = Color.LightSteelBlue;
			preset.UseShapeBackground2 = false;
			preset.ShapeBackground2 = Color.White;
			AddToList(preset);

			preset = new Preset();
			preset.Name = "Metalic";
			preset.UseGlassEffect = true;
			preset.GlassEffectType = GlassEffectType.Type2;
			preset.UsePenAsGlow = true;
			preset.GlowColor = Color.White;
			preset.ReflectionColor = Color.White;
			preset.UseAeroEffect = true;
			preset.Opacity = 50;
			preset.InnerOutlineColor = Color.White;
			preset.ShadeColor = Color.Black;
			preset.UseEmbossEffect = false;
			preset.UseSmoothShadowEffect = false;
			preset.DiagramBackground = Color.FromArgb(0xC0, 0xC0, 0xC0);
			preset.ShapeBorder = Color.DarkSlateGray;
			preset.ShapeBackground1 = Color.LightSteelBlue;
			preset.UseShapeBackground2 = false;
			preset.ShapeBackground2 = Color.White;
			AddToList(preset);

			preset = new Preset();
			preset.Name = "Neon Glow";
			preset.UseGlassEffect = true;
			preset.GlassEffectType = GlassEffectType.Type2;
			preset.UsePenAsGlow = true;
			preset.GlowColor = Color.White;
			preset.ReflectionColor = Color.White;
			preset.UseAeroEffect = true;
			preset.Opacity = 0;
			preset.InnerOutlineColor = Color.Black;
			preset.ShadeColor = Color.FromArgb(0x80, 0xFF, 0xFF);
			preset.UseEmbossEffect = false;
			preset.UseSmoothShadowEffect = false;
			preset.DiagramBackground = Color.Black;
			preset.ShapeBorder = Color.FromArgb(0x80, 0xFF, 0xFF);
			preset.ShapeBackground1 = Color.Black;
			preset.UseShapeBackground2 = false;
			preset.ShapeBackground2 = Color.White;
			AddToList(preset);

			preset = new Preset();
			preset.Name = "Gray glass";
			preset.UseGlassEffect = true;
			preset.GlassEffectType = GlassEffectType.Type3;
			preset.UsePenAsGlow = false;
			preset.GlowColor = Color.White;
			preset.ReflectionColor = Color.White;
			preset.UseAeroEffect = true;
			preset.Opacity = 40;
			preset.InnerOutlineColor = Color.White;
			preset.ShadeColor = Color.Black;
			preset.UseEmbossEffect = false;
			preset.UseSmoothShadowEffect = false;
			preset.DiagramBackground = Color.Gray;
			preset.ShapeBorder = Color.Black;
			preset.ShapeBackground1 = Color.LightCyan;
			preset.UseShapeBackground2 = true;
			preset.ShapeBackground2 = Color.FromArgb(0xA8, 0xBF, 0xBF);
			AddToList(preset);

			preset = new Preset();
			preset.Name = "Brown";
			preset.UseGlassEffect = true;
			preset.GlassEffectType = GlassEffectType.Type1;
			preset.UsePenAsGlow = false;
			preset.GlowColor = Color.White;
			preset.ReflectionColor = Color.White;
			preset.UseAeroEffect = false;
			preset.Opacity = 40;
			preset.InnerOutlineColor = Color.White;
			preset.ShadeColor = Color.Black;
			preset.UseEmbossEffect = false;
			preset.UseSmoothShadowEffect = true;
			preset.DiagramBackground = Color.PaleGoldenrod;
			preset.ShapeBorder = Color.Black;
			preset.ShapeBackground1 = Color.FromArgb(0x80, 0x90, 0x30, 0x20);
			preset.UseShapeBackground2 = false;
			preset.ShapeBackground2 = Color.White;
			AddToList(preset);

			preset = new Preset();
			preset.Name = "Bright Red";
			preset.UseGlassEffect = true;
			preset.GlassEffectType = GlassEffectType.Type4;
			preset.UsePenAsGlow = false;
			preset.GlowColor = Color.Black;
			preset.ReflectionColor = Color.White;
			preset.UseAeroEffect = false;
			preset.Opacity = 40;
			preset.InnerOutlineColor = Color.White;
			preset.ShadeColor = Color.Black;
			preset.UseEmbossEffect = false;
			preset.UseSmoothShadowEffect = false;
			preset.DiagramBackground = Color.White;
			preset.ShapeBorder = Color.FromArgb(0x8E, 0x8F, 0x8F);
			preset.ShapeBackground1 = Color.Red;
			preset.UseShapeBackground2 = false;
			preset.ShapeBackground2 = Color.White;
			AddToList(preset);
		}

		private void AddToList(Preset preset)
		{
			ListViewItem item = new ListViewItem();
			item.Text = preset.Name;
			item.Tag = preset;
			presetsList.Items.Add(item);
		}

		private void presetsList_ItemCheck(object sender, System.Windows.Forms.ItemCheckEventArgs e)
		{
			if (suspendUpdates)
				return;

			if (e.NewValue == CheckState.Checked)
			{
				// Uncheck all other items
				suspendUpdates = true;
				for (int i = 0; i < presetsList.Items.Count; i++)
				{
					if (i != e.Index)
						presetsList.Items[i].Checked = false;
				}
				suspendUpdates = false;

				ApplyPreset(presetsList.Items[e.Index].Tag as Preset);
			}
			else
			{
				ApplyPreset(defaultPreset);
			}
		}
		private bool suspendUpdates = false;

		private void checkBox1_CheckedChanged(object sender, EventArgs e)
		{
			if (checkBox1.Checked)
				diagram.NodeEffects.Add(glassEffect);
			else
				diagram.NodeEffects.Remove(glassEffect);
		}

		private void comboBox1_SelectedIndexChanged(object sender, EventArgs e)
		{
			glassEffect.Type = (GlassEffectType)comboBox1.SelectedIndex;
		}

		private void checkBox2_CheckedChanged(object sender, EventArgs e)
		{
			glassEffect.UsePenAsGlow = checkBox2.Checked;
		}

		private void colorPicker1_ColorChanged(object sender, EventArgs e)
		{
			glassEffect.GlowColor = colorPicker1.Color;
		}

		private void colorPicker2_ColorChanged(object sender, EventArgs e)
		{
			glassEffect.ReflectionColor = colorPicker2.Color;
		}

		private void checkBox3_CheckedChanged(object sender, EventArgs e)
		{
			if (checkBox3.Checked)
				diagram.NodeEffects.Add(aeroEffect);
			else
				diagram.NodeEffects.Remove(aeroEffect);
		}

		private void numericUpDown1_ValueChanged(object sender, EventArgs e)
		{
			aeroEffect.Opacity = (float)numericUpDown1.Value / 100;
		}

		private void colorPicker4_ColorChanged(object sender, EventArgs e)
		{
			aeroEffect.InnerOutlineColor = colorPicker4.Color;
		}

		private void colorPicker3_ColorChanged(object sender, EventArgs e)
		{
			aeroEffect.ShadeColor = colorPicker3.Color;
		}

		private void checkBox4_CheckedChanged(object sender, EventArgs e)
		{
			if (checkBox4.Checked)
				diagram.NodeEffects.Add(embossEffect);
			else
				diagram.NodeEffects.Remove(embossEffect);
		}

		private void checkBox5_CheckedChanged(object sender, EventArgs e)
		{
			if (checkBox5.Checked)
			{
				diagram.NodeEffects.Add(smoothShadowEffect);
				diagram.ShadowsStyle = ShadowsStyle.OneLevel;
			}
			else
			{
				diagram.NodeEffects.Remove(smoothShadowEffect);
				diagram.ShadowsStyle = ShadowsStyle.None;
			}
		}

		private void colorPicker5_ColorChanged(object sender, EventArgs e)
		{
			diagram.BackBrush = new MindFusion.Drawing.SolidBrush(colorPicker5.Color);
		}

		private void colorPicker7_ColorChanged(object sender, EventArgs e)
		{
			MindFusion.Drawing.SolidBrush stroke = new MindFusion.Drawing.SolidBrush(colorPicker7.Color);

			diagram.ShapeNodeStyle.Stroke = stroke;
		}

		private void colorPicker6_ColorChanged(object sender, EventArgs e)
		{
			UpdateShapeBrush();
		}

		private void checkBox6_CheckedChanged(object sender, EventArgs e)
		{
			UpdateShapeBrush();
			colorPicker8.Enabled = checkBox6.Checked;
		}

		private void colorPicker8_ColorChanged(object sender, EventArgs e)
		{
			UpdateShapeBrush();
		}

		private void UpdateShapeBrush()
		{
			MindFusion.Drawing.Brush brush = null;
			if (!checkBox6.Checked)
			{
				brush = new MindFusion.Drawing.SolidBrush(colorPicker6.Color);
			}
			else
			{
				brush = new MindFusion.Drawing.LinearGradientBrush(
					colorPicker6.Color, colorPicker8.Color, 90);
			}

			diagram.ShapeNodeStyle.Brush = brush;
		}

		private void ApplyPreset(Preset preset)
		{
			checkBox1.Checked = preset.UseGlassEffect;
			comboBox1.SelectedIndex = (int)preset.GlassEffectType;
			checkBox2.Checked = preset.UsePenAsGlow;
			colorPicker1.Color = preset.GlowColor;
			colorPicker2.Color = preset.ReflectionColor;

			checkBox3.Checked = preset.UseAeroEffect;
			numericUpDown1.Value = preset.Opacity;
			colorPicker3.Color = preset.ShadeColor;
			colorPicker4.Color = preset.InnerOutlineColor;

			checkBox4.Checked = preset.UseEmbossEffect;

			checkBox5.Checked = preset.UseSmoothShadowEffect;

			colorPicker5.Color = preset.DiagramBackground;
			colorPicker7.Color = preset.ShapeBorder;
			colorPicker6.Color = preset.ShapeBackground1;
			checkBox6.Checked = preset.UseShapeBackground2;
			colorPicker8.Color = preset.ShapeBackground2;
		}


		private GlassEffect glassEffect;
		private AeroEffect aeroEffect;
		private EmbossEffect embossEffect;
		private SmoothShadowEffect smoothShadowEffect;
		private Preset defaultPreset;

		private class Preset
		{
			public string Name;

			// Glass
			public bool UseGlassEffect;
			public GlassEffectType GlassEffectType;
			public bool UsePenAsGlow;
			public Color GlowColor;
			public Color ReflectionColor;

			// Aero
			public bool UseAeroEffect;
			public int Opacity;
			public Color InnerOutlineColor;
			public Color ShadeColor;

			// Emboss
			public bool UseEmbossEffect;

			// Shadow
			public bool UseSmoothShadowEffect;

			// Misc
			public Color DiagramBackground;
			public Color ShapeBorder;
			public Color ShapeBackground1;
			public bool UseShapeBackground2;
			public Color ShapeBackground2;
		}
	}
}